/**
Copyright © 1997-2016, Stephen Gose LLC. All rights reserved.
Visit our game show case:	https://www.renown-games.com 
License information:		https://makingbrowsergames.com/starterkits/
Affiliate Information:		https://www.stephen-gose.com/affiliates-program/

 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER  
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN  
 * THE SOFTWARE.
 * 
 *
 * File Name: Main.js
 * Live Demo: https://makingbrowsergames.com/starterkits/xxxxxxx
 * Source Code: https://makingbrowsergames.com/book/
 * Description: File for controlling and displaying game scenes;
 *               managing global variables throughout game state.

 * Author: Stephen Gose
 * Version: 0.0.0.19971027-1  //semantic version: by (0.0.0.date-#)
 * Phaser Version: 3.24+
 * Author URL: https://www.stephen-gose.com/
 * Support: support@pbmcube.com
 *
 * \u00A9 Copyright ©  1974-2017 Stephen Gose LLC. All rights reserved. 
 * Redistribution of part or whole of this file and
 *   the accompanying files is strictly prohibited.
 * 
 * Do not sell! Do not distribute!  
 *  Please refer to the Terms of Use and 
 *    End Users License Agreement (EULA).
 *  This is a licensed file with EULA permission. 
 *  
 * Search for "//**TODO**" to tailor this file for your production; 
 *	 doing so will void any support agreement.
 *
 * Construction References:
 * https://www.adequatelygood.com/JavaScript-Module-Pattern-In-Depth.html
 * https://labs.phaser.io/edit.html?src=src\scenes\tutorial\scene%20controller.js
 * https://labs.phaser.io/edit.html?src=src\scenes\changing%20scene.js
 * https://www.html5gamedevs.com/topic/37199-extending-phaser-classes-es6-syntax-vs-phaserclass-differences/
 * 
 */
 
"use strict";
// =========================================================
/**	game set-up	
 *  Beginning of Static internal game database(s)
 *  //**TODO** 
 *  NOTE: the following data structures are for development only.
 *  The optimum usage is a local or remote database using PouchDB or SQLite.
 *  Stay away from IndexDB since it is deprecated.
 * 
 * -------------------------------------------
 * Anti-Pattern: global name-space pollution by
 *   Standard Phaser v3.x.x Scene preparations
 * init:     function init() {},    //initial game phase data  
 * preload:  function preload() {}, //queue & download game assets 
 * create:   function create() {},  //make cached assets available 
 * update:   function update() {},  //begin the game loop  
 * render:   function render() {},  //render current display 
 * shutdown: function shutdown() {} //close and garbage collection? 
 * -------------------------------------------
 */

// =========================================================
// Acknowledgments:
// =========================================================
console.log ("%c %c %c|    Starting my awesome MMoG game Prototype!       |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|     Phaser 3 Game Prototyping Starter Kit         |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|      Copyright \u00A9 1974-2017, Stephen Gose.         |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|               All rights reserved.                |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|  Book available at: https://leanpub.com/p3gskc    |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|  \u2665\u2665\u2665\u2665\u2665 -> $1,420 Licenses included in book!       |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");

function gameMechanics(){
	
	console.log("%c %c %c|              Game mechanics launched             |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
	console.log("Game Mechanics === Ext? "+Object.isExtensible( GAMEAPP ));
	//console.log(Object.values(GAMEAPP));
	//console.log(Object.getPrototypeOf(GAMEAPP));
	console.log(Object.getOwnPropertyDescriptors(GAMEAPP));

};

// Beginning of Static databases (English only)
// =========================================================

// 
// =========================================================
// End of database structures
//**TODO** 
// NOTE: the following data structures are for development only.
//	The optimum usage is a local or remote database using PouchDB or SQLite.
//	Stay away from IndexDB since it is deprecated.
// =========================================================

// Game global config object
// =========================================================
// creates our Phaser III Game configurations. 
// JS object literals are NOT hoisted.	
var config = {
	//dozens of configurations parameters available.  
	type: Phaser.AUTO,  // .WEBGL or .Canvas  
	width:    800,		// x width - best results use Golden Ratio
	height:   500,		// y width - best results use Golden Ratio
	
	title: 'Phaser3 Game Prototyping Starter Kit     \n',	//Game Title
	url: '\nhttps://makingbrowsergames.com/p3gp-book/',		//Game URL location
	//https://semver.org/spec/v1.0.0.html
	version: 'ersion  1.0.0.0 (semver) 20170215       \n',	
	input: {
		gamepad: false,	// not interested in game pad input
		keyboard: true,
		mouse: true,
		touch: true,
		// Breaking Change: v3.24+
		// queue: true	// use pre-v3.16 legacy Input Manager event queue
	},
	backgroundColor: 0, //Custom RGB color. (0x000 - 0xFFF)
	physics: {
		default: 'arcade'
	}, 
    /*
	dom: {
        createContainer: true
    },
    */
	scene:  [],
	//parent: document.body,
	parent:   "game-area",
	pixelArt: false,
	antialias: true	
};

/**
 * Standard global pollution in Phaser v3.x.x preparations
 * Configuration as an object literal: NOT HOISTED
//Using function method; JS Hoisted!
var config = function config() {
		var width = window.GAMEAPP.width,
		height = window.GAMEAPP.height,
		type = Phaser.AUTO,
		parent = gameCanvas;
	};
*/	
console.log("Configure Obj: === Ext? "+Object.isExtensible( config ));
//console.log(Object.values(config));	
//console.log(Object.getPrototypeOf(config));
console.log(Object.getOwnPropertyDescriptors(config));
// End global config object
// =========================================================
// **TODO** Creating Game name-space called GAMEAPP; 
// refactor GAMEAPP with your bespoke project name
var game = {};
// creates a game name-space
// =========================================================
//**TODO** Creating Game name-space called "GAMEAPP" object;
// rename "GAMEAPP" with your project's name
var GAMEAPP = {
	//US Copr. or Copyright; UTF8 "circled c" is \u00A9 equivalent to &copy;
	Copr: "\u00A9 Copyright © 1974-2016, Stephen Gose. All rights reserved.\n",
	
	//assign a unique identification (SKU code) for local storage
	gDefDelim: "game SKU",	//See cookie/local.storage below
	
	// If there's music in your game, and it needs to play through-out a 
	//	  few State swaps, then you could reference it below. 
	//music: null,
	
	// Toggle background music theme on or off; starts in "on/true" state 
	//musicToggle: true,
	
	// Your game can check MYGAMEAPP.orientated in the game loops 
	//	  to know if it should pause or not.
	//orientated: false,
	
	//Canvas dimensions: world and viewport Height and Width
	//**TODO** adjust for your game deployment
	viewportWidth:	800,	//game view using Golden Ration
	viewportHeight:	500,
	worldWidth:		800,	//world view using Golden Ration
	worldHeight:	500,
	
	//==================================================
	// -------------------------------------------
	//	Trash Global Variables (alphabetical)
	// -------------------------------------------
	//**TODO** 
	//  Refactor and adjust for your own game deployment
	//	Here we have some global level vars that persist regardless of State. 
	//==================================================
	gameLanguage: 0,	//Default is English (American)
	InfoText: "Default Text.",	
	//button styling
	styleBTN: { font: "26px Arial", fill: "#000000", align: "center" },
	
	//Combat HUD styling
	//	hero text
	styleCmbtH: { font: "9px Arial", fill: "#66ffff", align: "center" },	
	styleNarrH: { font: "9px Arial", fill: "#66ffff", align: "left" },
	//	monster text
	styleCmbtM: { font: "9px Arial", fill: "#66ff66", align: "center" },
	styleNarrM: { font: "9px Arial", fill: "#66ff66", align: "left" },
	
	//HUD styling
	styleHUD: { font: "14px Arial", fill: "#ff9900", align: "center" },
	//Room Alert styling
	styleRA: { font: "18px Arial", fill: "#ff9900", align: "center" },
	//Dynamic Story style
	styleStory: { font: "16px Arial", fill: "#ff9900", align: "left" },
	//toolTip styling
	styleTT: { font: "11px Arial", fill: "#ff9900", align: "center" },
	
	// =========================================================
	// here we will store all game phase/states
	//  state object filled as js/ files load.
	state: {},
	
	// =========================================================
	// -------------------------------------------
	// Main game Handler methods
	// -------------------------------------------
	//**TODO**:
	//	refactor and adjust for your game deployment
	//	remove console debug information on public deployment
	// =========================================================
	main: function(){
		
		//Phaser v3.15.x+
		game = new Phaser.Game(config);
		console.log("Game prototype (Phaser.Game): Ext? "+Object.isExtensible( Phaser ));
		//console.log(Object.values(Phaser));
		//console.log(Object.getPrototypeOf(Phaser));
		console.log(Object.getOwnPropertyDescriptors(Phaser));

		// add all game states
		for( var stateName in window.GAMEAPP.state ){
			console.log("Crnt State: "+stateName);
			game.scene.add(stateName, window.GAMEAPP.state[stateName]);
		}

		//Dynamically assigned Game Mechanics from JSON Data file.
		gameMechanics();		

		//move to next game phase
		console.log("Leaving GAMEAPP.main -> boot");	//debug
		game.scene.start('boot');
	},
	// =========================================================
	/**
	 *  NEW Phaser v3.x.x method: passing via Object.create
	 *      configuration submitted as external object
	 *  
	 *   game = Object.create(Phaser);
	 *  
	 *  Phaser docs suggests a separate object literal config = {...}; 
	 *  See "Phaser 3 Game Prototyping" book reference  
	 *    for all the game configuration parameters. 
	 *  Available from 
	 *    https://leanpub.com/phaser3gameprototyping or search Amazon.com 
	 *  
	 * Experiment: Embed Config into Phaser v3.x.x directly. 
	 *  Note we could have simply inserted the object directly.
	 *  You'll occasionally find this methods in the Phaser3 lab examples.
	 *  https://labs.phaser.io/
	 *  

		this.game = new Phaser.Game(
			//internalized configuration object
			// One less global pollution!
			{
				width: window.GAMEAPP.viewportWidth,
				height: window.GAMEAPP.viewportHeight,
				type: Phaser.AUTO,
				scene: window.GAMEAPP.state,
				parent: gameCanvas	//or document.body
				
			});
	*/
	
	// =================================================
	// -------------------------------------------
	// Supporting game Function & Classes
	// -------------------------------------------
	//**TODO**:
	//	Change name-space from generic GAMEAPP to your project 
	//	refactor and adjust for your game deployment
	//	remove console debug information on public deployment
	// =========================================================
	_audioMgr: function(mode, game) {
		switch(mode) {
			case 'init': {
				GAMEAPP.Storage.initUnset('GAMEAPP.audio', true);
				GAMEAPP._audioStatus = GAMEAPP.Storage.get('GAMEAPP.audio');
				// GAMEAPP._soundClick = game.add.audio('audio-click');
				GAMEAPP._sound = [];
				GAMEAPP._sound['click'] = game.add.audio('audio-click');
				if(!GAMEAPP._soundMusic) {
					GAMEAPP._soundMusic = game.add.audio('audio-theme',1,true);
					GAMEAPP._soundMusic.volume = 0.5;
				}
				break;
			}
			case 'on': {
				GAMEAPP._audioStatus = true;
				break;
			}
			case 'off': {
				GAMEAPP._audioStatus = false;
				break;
			}
			case 'switch': {
				GAMEAPP._audioStatus =! GAMEAPP._audioStatus;
				break;
			}
			default: {}
		}
		if(GAMEAPP._audioStatus) {
			GAMEAPP._audioOffset = 0;
			if(GAMEAPP._soundMusic) {
				if(!GAMEAPP._soundMusic.isPlaying) {
					GAMEAPP._soundMusic.play('',0,1,true);
				}
			}
		}
		else {
			GAMEAPP._audioOffset = 4;
			if(GAMEAPP._soundMusic) {
				GAMEAPP._soundMusic.stop();
			}
		}
		GAMEAPP.Storage.set('GAMEAPP.audio',GAMEAPP._audioStatus);
		game.buttonAudio.setFrames(GAMEAPP._audioOffset+1, GAMEAPP._audioOffset+0, GAMEAPP._audioOffset+2);
	},

	// 
	// =================================================
	//This could be loaded externally from a JSON file
	//I choose to have this internalized to avoid additional downloads.
	// When loading a file from a different domain, the game must have 
	//  a 'Access-Control-Allow-Origin' header present.
	_assignLanguage: function(){
		
		switch(GAMEAPP.gameLanguage){
			case 0:	//"english":
				console.log("English selected");
				GAMEAPP.credittxt= "Credits & Info";
				GAMEAPP.donatxt= "How did you make this?";
				GAMEAPP.helptxt= "Help & Instructions";
				GAMEAPP.introPara= "Introduction game-text here.\n";
				GAMEAPP.moretxt= "More games";
				GAMEAPP.opttxt= "Options";
				GAMEAPP.playtxt= "Start Game";
				GAMEAPP.sharetxt= "Share with friends";
				GAMEAPP.webtxt= "Webmaster License Info";
				break;
			
			case 1: //"china"
				GAMEAPP.credittxt = "片尾";
				GAMEAPP.donatxt = "";
				GAMEAPP.helptxt = "游戏说明";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "更多的游戏";
				GAMEAPP.opttxt = "游戏选项";
				GAMEAPP.playtxt = "玩这个游戏";
				GAMEAPP.sharetxt = "分享此游戏";
				GAMEAPP.webtxt =  "";
				break;
			
			case 2:	//"denmark":
				GAMEAPP.credittxt = "afsluttende credits";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "spil instruktioner";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "flere spil";
				GAMEAPP.opttxt = "spilindstillinger";
				GAMEAPP.playtxt = "spille dette spil";
				GAMEAPP.sharetxt = "deler dette spil";
				GAMEAPP.webtxt =  "";	
				break;
			
			case 3:	//"french":
				GAMEAPP.credittxt = "générique de fin";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Instructions du jeu";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "plus de jeux de";
				GAMEAPP.opttxt = "options de jeu";
				GAMEAPP.playtxt = "Jouer à ce jeu";
				GAMEAPP.sharetxt = "partager ce jeu";
				GAMEAPP.webtxt =  "";
				break;

			case 4:	//"german":
				GAMEAPP.credittxt = "Abspann";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Anweisungen";
				GAMEAPP.introPara =""; 
				GAMEAPP.moretxt = "mehr";
				GAMEAPP.opttxt = "Spieloptionen";
				GAMEAPP.playtxt = "Spiel";
				GAMEAPP.sharetxt = "Aktien dieser";
				GAMEAPP.webtxt =  "";
				break;	
				
			case 5:	//"italian":
				GAMEAPP.credittxt = "Associare";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Istruzioni di gioco";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "Quantità";
				GAMEAPP.opttxt = "Opzioni del gioco";
				GAMEAPP.playtxt = "Giocare questo gioco";
				GAMEAPP.sharetxt = "Condividere questo gioco";
				GAMEAPP.webtxt =  "";
				break;
			
			case 6:	//"japanese":
				GAMEAPP.credittxt = "閉じるクレジット";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "ゲームの説明";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "より多くのゲーム";
				GAMEAPP.opttxt = "ゲームオプション";
				GAMEAPP.playtxt = "このゲームをプレイ";
				GAMEAPP.sharetxt = "共有このゲーム";
				GAMEAPP.webtxt =  "";
				break;
			
			case 7:	//"netherlands":
				GAMEAPP.credittxt = "Credits";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Spelregels";
				GAMEAPP.introPara =  "";
				GAMEAPP.moretxt = "meer games";
				GAMEAPP.opttxt = "spelopties";
				GAMEAPP.playtxt = "dit spel";
				GAMEAPP.sharetxt = "aandeel dit spel";
				GAMEAPP.webtxt =  "";
				break;
			
			case 8:	//"portugal":
				GAMEAPP.credittxt = "créditos finais";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Instruções de jogo";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "mais jogos";
				GAMEAPP.opttxt = "opções de jogo";
				GAMEAPP.playtxt = "Jogar este jogo";
				GAMEAPP.sharetxt = "Compartilhe este jogo";
				GAMEAPP.webtxt =  "";
				break;
			
			case 9:	//"spain":
				GAMEAPP.credittxt = "Créditos";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Instrucciones";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "Más";
				GAMEAPP.opttxt = "Opciones de juego";
				GAMEAPP.playtxt = "Jugar a este juego";
				GAMEAPP.sharetxt = "Compartir esta";
				GAMEAPP.webtxt = "";		
				break;
			
			case 10:	//"sweden":
				GAMEAPP.credittxt = "eftertexterna";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "instruktioner";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "mer";
				GAMEAPP.opttxt = "Spelalternativ";
				GAMEAPP.playtxt = "Spela";
				GAMEAPP.sharetxt = "VARA DELAKTIG I";
				GAMEAPP.webtxt =  "";
				break;
			
			default:
				console.log("English selected");
				GAMEAPP.credittxt= "Credits & Info";
				GAMEAPP.donatxt= "Support Us";
				GAMEAPP.helptxt= "Help & Instructions";
				GAMEAPP.introPara= "";
				GAMEAPP.moretxt= "More games";
				GAMEAPP.opttxt= "Options";
				GAMEAPP.playtxt= "Start Game";
				GAMEAPP.sharetxt= "Share with friends";
				GAMEAPP.webtxt= "Webmaster License Info";
				
				break;
		}
	},	

	// 
	// =================================================
	// -------------------------------------------
	// End Main game Handler
	// -------------------------------------------
	// =================================================
};

console.log("Game obj: === Ext? "+Object.isExtensible( GAMEAPP ));
//console.log(Object.values(GAMEAPP));
//console.log(Object.getPrototypeOf(GAMEAPP));
console.log(Object.getOwnPropertyDescriptors(GAMEAPP));


/** 
 *  DEPRECATED METHOD - NEVER EVER USE THIS AGAIN!
 * 	See "Phaser III Game Design Workbook" for complete explanation
 *	https://leanpub.com/phaser3gamedesignworkbook
 *
 * 	window.onload = function () {
 *	    let game =  . . . . . .
 *	};
*/


//preferred game launch method for BOM.
document.addEventListener('DOMContentLoaded', function(){
	window.GAMEAPP.main();
}, false);

console.log("Game prototype (Phaser.Game): === Ext? "+Object.isExtensible( Phaser ));
//console.log(Object.values(Phaser));
//console.log(Object.getPrototypeOf(Phaser));
console.log(Object.getOwnPropertyDescriptors(Phaser));

//
// =================================================
/** End of file */
/** Location: ./js/main.js */
  